# This file is placed in the Public Domain.
#
#
# pylint: disable=C,R,W1503


"no tests"


import unittest



class TestNone(unittest.TestCase):

    def test_nope(self):
        self.assertTrue(True)
